/// <reference types="node" />

import { Context, ILogger, Identity, NextFunction, ObjectId, Resolvable, TypeRegistry } from '@e2/core';
import { Index, MongoApplicationSettings, MongoClient, MongoCollection, MongoDatabase, MongoModel } from '@e2/mongodb';
import { HttpError, InternalWebContext, WebApplication, WebApplicationSettings, WebContext, WebController } from '@e2/web';
import { Constructor, IAttribute, IInterceptor, IInvocation } from 'agentframework';
import { IncomingMessage } from 'http';

export declare class IdentityProfile {
	_id: ObjectId | null;
	displayName: string;
	displayImageURL: string;
}
export interface IActivatable {
	activated?: boolean;
	activatedAt?: Date;
	activatedFrom?: string;
	activatedBy?: IdentityProfile;
	activatedReason?: string;
}
export interface IArchivable {
	archived?: boolean;
	archivedAt?: Date;
	archivedFrom?: string;
	archivedBy?: IdentityProfile;
	archivedReason?: string;
}
export interface IAuditable {
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
}
export interface IBlockable {
	blocked?: boolean;
	blockedAt?: Date;
	blockedFrom?: string;
	blockedBy?: IdentityProfile;
	blockedReason?: string;
}
export interface IChainable {
	previous?: ObjectId;
}
export interface IChargeable {
	charge?: number;
	chargedAt?: Date;
	chargedFrom?: string;
	chargedBy?: IdentityProfile;
	chargedReason?: string;
	dischargedAt?: Date;
	dischargedFrom?: string;
	dischargedBy?: IdentityProfile;
	dischargedReason?: string;
}
export interface IDeletable {
	deleted?: boolean;
	deletedAt?: Date;
	deletedFrom?: string;
	deletedBy?: IdentityProfile;
	deletedReason?: string;
}
export interface IExpirable {
	expireAt?: Date;
}
export interface IMutable {
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
}
export interface IEdge extends IAuditable, IMutable, IExpirable {
	outType: string;
	label: string;
	inType: string;
	outV: string | any;
	inV: string | any;
	meta: any;
	expireAt: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
}
export interface INameable {
	name: string;
	displayName: string;
}
export interface IProfile {
	displayName: string;
	displayImageURL: string;
}
export interface IStateful {
	state: string;
}
export interface IVerifiable {
	verified?: boolean;
	verifiedAt?: Date;
	verifiedFrom?: string;
	verifiedBy?: IdentityProfile;
	verifiedReason?: string;
	verifyToken?: string;
	verifyTokenFailures?: number;
	verifyTokenExpires?: Date;
}
export interface IVertex extends IAuditable, IMutable, IExpirable {
}
export declare const SYSTEM_MAX_UPLOAD_SIZE: number;
export declare const SYSTEM_MAX_APFS_CHUNK_SIZE: number;
export declare const SYSTEM_MAX_NUMBER_OF_ACCOUNTS_PER_USER = 1;
export declare class WebFoundationSettings extends WebApplicationSettings implements MongoApplicationSettings {
	ADMIN_EMAIL: string;
	OWNER_NAME: string;
	OWNER_COMPANY_NAME: string;
	OWNER_COMPANY_CODE: string;
	OWNER_COMPANY_ADDRESS: Array<string>;
	OWNER_COMPANY_TELEPHONE: string;
	OWNER_COMPANY_FAX: string;
	OWNER_AGREEMENTS: Array<any>;
	MONGODB_URL: string;
	MONGODB_DATABASE: string;
	MONGODB_EVENT_LOG_LEVEL: string;
	WEB_SESSION_COOKIE_KEY: string;
	WEB_SESSION_COOKIE_SECRET: Array<string>;
	WEB_SESSION_EXPIRES_IN_MILLISECONDS: number;
	WEB_TOKEN_EXPIRES_IN_MILLISECONDS: number;
	WEB_INVITATION_EXPIRES_IN_MILLISECONDS: number;
	WEB_EMAIL_VERIFICATION_RESEND_COOLDOWN_IN_MILLISECONDS: number;
	WEB_EMAIL_VERIFICATION_EXPIRES_IN_MILLISECONDS: number;
	WEB_SIGNIN_PATH: string;
	WEB_SIGNIN_SUCCESS_PATH: string;
	WEB_SIGNIN_FAILED_PATH: string;
	WEB_ACCESS_DENIED_PATH: string;
	WEB_SIGNOUT_PATH: string;
	WEB_SIGNOUT_COMPLETE_PATH: string;
	MAX_NUMBER_OF_ACCOUNTS_PER_USER: number;
	MAX_UPLOAD_SIZE: number;
	MAX_APFS_CHUNK_SIZE: number;
	JSONSDK_ACCESS_KEY: string;
	OIDC_AUTHENTICATION_EXPIRES_IN_MILLISECONDS: number;
	OIDC_REDIRECT_LIST: Array<string>;
	OIDC_NO_REDIRECTS_LIST: Array<string>;
	OIDC_MICROSOFT: boolean;
	OIDC_MICROSOFT_ISSUER_URL: string;
	OIDC_MICROSOFT_CLIENT_ID: string;
	OIDC_MICROSOFT_CLIENT_SECRET: string;
	OIDC_MICROSOFT_SCOPES: Array<string>;
	OIDC_GOOGLE: boolean;
	OIDC_GOOGLE_ISSUER_URL: string;
	OIDC_GOOGLE_CLIENT_ID: string;
	OIDC_GOOGLE_CLIENT_SECRET: string;
	OIDC_GOOGLE_SCOPES: Array<string>;
}
declare class SecuredCookie extends Resolvable<WebFoundationSettings> {
	private readonly crypts;
	private readonly base72;
	constructor(app: Context<WebFoundationSettings>);
	sign(data: string): string;
	verify(signed: string): string | false;
}
declare class SessionCookieManager extends Resolvable<WebFoundationSettings> {
	security: SecuredCookie;
	get(ctx: WebContext<WebFoundationSettings>): string | null;
	set(ctx: WebContext<WebFoundationSettings>, data: string): void;
	clear(ctx: WebContext<WebFoundationSettings>): void;
}
export interface Session {
	session: ObjectId | null;
	user: ObjectId | null;
	token: ObjectId | null;
	roles: Array<string>;
	groups: Array<string>;
	scope: Array<string>;
	profile: IdentityProfile | null;
}
declare class InternalSymbol {
	static readonly ServiceContext: unique symbol;
}
export interface FetchConfiguration {
	method: string;
	url: string;
	data: any;
	responseType: string;
	credentials: boolean;
}
export interface FetchResponse {
	ok: boolean | number;
	headers: any;
	status: number;
	error?: string;
	errcode?: number;
	message?: string;
	stack?: string;
	result?: any;
	results?: Array<any>;
	data?: Buffer;
	stream?: IncomingMessage;
}
export interface FetchParameters {
	url: string | undefined;
	init: FetchConfiguration | undefined;
}
export declare class Fetch extends WebFoundationUserService<WebFoundationSettings> {
	fetch(url: string, init?: FetchConfiguration): Promise<FetchResponse>;
	fetch(init: FetchConfiguration): Promise<FetchResponse>;
	private buildAuthorisedHeaders;
}
export declare class WebIdentity implements Identity {
	id: string;
	address: string;
	session: ObjectId | null;
	user: ObjectId | null;
	roles: Array<string>;
	groups: Array<string>;
	scope: Array<string>;
	token: ObjectId | null;
	profile: IdentityProfile | null;
}
export interface WebFoundationToken {
}
export interface WebFoundationDomain<T extends WebFoundationSettings> extends ILogger {
	readonly identity: WebIdentity;
	readonly token: WebFoundationToken;
	readonly settings: T;
	set(key: string, value: any): any;
	has(key: string): boolean;
	fetch(url: string, init?: Partial<FetchConfiguration>): Promise<FetchResponse>;
	fetch(init: Partial<FetchConfiguration>): Promise<FetchResponse>;
	construct<T>(type: Constructor, transit?: boolean): T;
	resolve<T>(type: Constructor, transit?: boolean): Promise<T>;
	collection<T>(type: Constructor): T;
}
export declare abstract class WebFoundationService<T extends WebFoundationSettings> extends Resolvable<T> {
}
export declare abstract class WebFoundationUserService<T extends WebFoundationSettings> extends Resolvable<T> {
	private [InternalSymbol.ServiceContext];
	constructor(app: WebFoundationApplication<T>, ctx: any);
	readonly domain: WebFoundationDomain<T>;
	addResponseHeader(name: string, value: number | string): void;
	getRequestHeader(name: string): string | string[] | undefined;
}
declare abstract class WfService extends WebFoundationService<WebFoundationSettings> {
}
export declare class WebSessionService extends WfService {
	readonly name: string;
	sessionCookie: SessionCookieManager;
	create(ctx: WebContext<WebFoundationSettings>, id?: string): Partial<Session> | null;
	update(ctx: WebContext<WebFoundationSettings>, session: Partial<Session>): void;
	clear(ctx: WebContext<WebFoundationSettings>): void;
}
export declare class WebFoundationApplication<T extends WebFoundationSettings> extends WebApplication<T> {
	WebIdentityService: WebSessionService;
	protected createRequestState(ctx: InternalWebContext<T>, state?: any): any;
	protected createRequestIdentity(ctx: InternalWebContext<T>): Identity;
	protected createRequestSession(ctx: InternalWebContext<T>): Partial<Session> | null;
	onInitApplication(): void;
}
export interface WfCtx extends WebFoundationContext<WebFoundationSettings> {
}
export interface WfApp extends WebFoundationApplication<WebFoundationSettings> {
}
export declare class Graph extends MongoDatabase {
	readonly identity: Partial<WebIdentity>;
	constructor(client: MongoClient, app: WfApp, ctx: WfCtx);
	audit(action: string, updates?: object, status?: boolean): object;
}
export interface IUserAddress {
	address: string;
	postalCode: string;
	city: string;
	state: string;
	countryCode: string;
	country: string;
}
export interface IUserSettings {
	MAX_NUMBER_OF_ACCOUNTS_PER_USER: number;
	MAX_UPLOAD_SIZE: number;
	MAX_STORAGE_SIZE: number;
}
export interface IUser {
	displayName: string;
	displayImageURL: string;
	signature?: any;
	location?: any;
	address: IUserAddress;
	email?: string;
	mobile?: string;
	roles: Array<string>;
	identities: any;
	settings: IUserSettings;
}
export declare class Collection<T extends Model> extends MongoCollection<T> {
}
export interface ModelConstructor<T extends Model> {
	new (): T;
	getType(): string;
}
export declare class Model extends MongoModel implements IAuditable, IMutable, IExpirable {
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	expireAt: Date;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	collection<T extends Model>(): Collection<T>;
}
export declare class EdgeModel extends Model implements IEdge {
	outType: string;
	label: string;
	inType: string;
	outV: ObjectId | any;
	inV: ObjectId | any;
	meta: any;
	expireAt: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
}
export declare class VertexModel extends Model implements IVertex {
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	expireAt: Date;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	$edge(): string | false;
	addEdgeFrom(label: string, outModel: Model, meta?: any): Promise<EdgeModel | null>;
	addEdgeTo(label: string, inModel: Model, meta?: any): Promise<EdgeModel | null>;
	fetch(): Promise<Model>;
	findEdges(): Promise<Array<EdgeModel>>;
	findOutEdges(): Promise<Array<EdgeModel>>;
	findInEdges(): Promise<Array<EdgeModel>>;
	findFromVertices<T extends Model>(outCollection: Collection<T>, label?: string): Promise<Array<T>>;
	findToVertices<T extends Model>(inCollection: Collection<T>, label?: string): Promise<Array<T>>;
	findOutVertices<T extends Model>(outCollection: Collection<T>, label?: string): Promise<Array<T>>;
	findInVertices<T extends Model>(inCollection: Collection<T>, label?: string): Promise<Array<T>>;
	findOutTypes(label: string): Promise<Array<string>>;
	findInTypes(label: string): Promise<Array<string>>;
	findOutLabels(inModelConstructor: ModelConstructor<Model>): Promise<Array<string>>;
	findInLabels(inModelConstructor: ModelConstructor<Model>): Promise<Array<string>>;
}
export declare class UserModel extends VertexModel implements IUser, IActivatable, IBlockable, IDeletable, IProfile {
	displayName: string;
	displayImageURL: string;
	signature?: any;
	location?: any;
	address: IUserAddress;
	email?: string;
	mobile?: string;
	roles: Array<string>;
	identities: any;
	settings: IUserSettings;
	expireAt: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	activated?: boolean;
	activatedAt?: Date;
	activatedFrom?: string;
	activatedBy?: IdentityProfile;
	activatedReason?: string;
	blocked?: boolean;
	blockedAt?: Date;
	blockedFrom?: string;
	blockedBy?: IdentityProfile;
	blockedReason?: string;
	deleted?: boolean;
	deletedAt?: Date;
	deletedFrom?: string;
	deletedBy?: IdentityProfile;
	deletedReason?: string;
}
export interface WebFoundationContextState {
	uid: string;
	db: Graph;
	user?: UserModel;
	[x: string]: any;
}
export interface WebFoundationContext<T extends WebFoundationSettings> extends WebContext<T> {
	identity: WebIdentity;
	state: WebFoundationContextState;
	domain: WebFoundationDomain<T>;
}
export declare abstract class WebFoundationController<T extends WebFoundationSettings> extends WebController<T> {
	protected getFailureURL(ctx: WfCtx, err: Error): Promise<string>;
}
export declare class DatabaseError extends HttpError {
	readonly statusCode: number;
	readonly statusMessage: string;
}
export declare class ValidationError extends HttpError {
	readonly statusCode: number;
	readonly statusMessage: string;
}
export declare function user(): MethodDecorator;
export declare function admin(): MethodDecorator;
export declare class RoleAttribute implements IAttribute, IInterceptor {
	private role;
	constructor(role: string);
	beforeDecorate(target: Object | Function, targetKey?: string | symbol, descriptor?: PropertyDescriptor): boolean;
	getInterceptor(): IInterceptor;
	intercept(target: IInvocation, parameters: ArrayLike<any>): any;
}
export declare class Time {
	static duration(from: Date, to: Date): string;
}
export interface FormatterOptions {
	format: 'time' | 'percentage' | 'byte' | 'currency' | 'number';
	format_currency_code?: string;
	start?: number;
	end?: number;
}
export declare function formatTime(num: number): string;
export declare function formatNumber(num: number, options: FormatterOptions): string;
export declare class EdgeCollection extends Collection<EdgeModel> {
	constructor(db: Graph);
	outV(outV: ObjectId, label: string, filter?: any, projection?: any, sort?: any): Promise<Array<EdgeModel>>;
	inV(inV: ObjectId, label: string, outType: string, filter?: any, projection?: any, sort?: any): Promise<Array<EdgeModel>>;
	removeRelationships(target: ObjectId): Promise<Array<EdgeModel>>;
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
}
export declare class Validator {
	static validate(model: Model & IDeletable & IBlockable): void;
}
export declare class EmailModel extends VertexModel {
	name: string;
	displayName: string;
	user: ObjectId;
	verified: boolean;
	verifiedAt: Date;
	verifiedBy: IdentityProfile;
	verifiedFrom: string;
	verifyToken: string;
	verifyTokenFailures: number;
	verifyTokenExpires: Date;
	salt: string;
	password: string;
	resetPasswordAt: Date;
	resetPasswordToken: string;
	resetPasswordTokenExpires: Date;
	blocked: boolean;
	blockedAt: Date;
	blockedFrom: string;
	blockedBy: IdentityProfile;
	blockedReason: string;
	deleted: boolean;
	deletedAt: Date;
	deletedFrom: string;
	deletedBy: IdentityProfile;
	deletedReason: string;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	expireAt: Date;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
}
export declare class EmailCollection extends Collection<EmailModel> {
	constructor(db: Graph);
	static getIndices(): Array<any>;
	createIndices(): Promise<any>;
}
export declare class MobileModel extends VertexModel {
	name: string;
	displayName: string;
	salt: string;
	password: string;
	resetPasswordAt?: Date;
	resetPasswordFrom?: string;
	resetPasswordBy?: IdentityProfile;
	resetPasswordToken?: string;
	resetPasswordTokenFailures?: number;
	resetPasswordTokenExpires?: Date;
	verified?: boolean;
	verifiedAt?: Date;
	verifiedFrom?: string;
	verifiedBy?: IdentityProfile;
	verifiedReason?: string;
	verifyToken?: string;
	verifyTokenFailures?: number;
	verifyTokenExpires?: Date;
	expireAt: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	activated: boolean;
	activatedAt: Date;
	activatedFrom: string;
	activatedBy: IdentityProfile;
	blocked: boolean;
	blockedAt: Date;
	blockedFrom: string;
	blockedBy: IdentityProfile;
	blockedReason: string;
	deleted: boolean;
	deletedAt: Date;
	deletedFrom: string;
	deletedBy: IdentityProfile;
	deletedReason: string;
}
export declare class MobileCollection extends Collection<MobileModel> {
	constructor(db: Graph);
	static getIndices(): Array<any>;
	createIndices(): Promise<any>;
}
export interface OpenIdProviderConfigurationMandatoryFields {
	issuer: string;
	authorization_endpoint: string;
	jwks_uri: string;
	response_types_supported: Array<string>;
	subject_types_supported: Array<string>;
	token_endpoint: string;
	id_token_signing_alg_values_supported: Array<string>;
	scopes_supported: Array<string>;
	claims_supported: Array<string>;
}
export interface OpenIdProviderConfigurationOptionalFields {
	userinfo_endpoint?: string;
	registration_endpoint?: string;
	response_modes_supported?: Array<string>;
	grant_types_supported?: Array<string>;
	acr_values_supported?: Array<string>;
	id_token_encryption_alg_values_supported?: Array<string>;
	id_token_encryption_enc_values_supported?: Array<string>;
	userinfo_signing_alg_values_supported?: Array<string>;
	userinfo_encryption_alg_values_supported?: Array<string>;
	userinfo_encryption_enc_values_supported?: Array<string>;
	request_object_signing_alg_values_supported?: Array<string>;
	request_object_encryption_alg_values_supported?: Array<string>;
	request_object_encryption_enc_values_supported?: Array<string>;
	token_endpoint_auth_methods_supported?: Array<string>;
	token_endpoint_auth_signing_alg_values_supported?: Array<string>;
	display_values_supported?: Array<string>;
	claim_types_supported?: Array<string>;
	service_documentation?: string;
	claims_locales_supported?: Array<string>;
	ui_locales_supported?: Array<string>;
	claims_parameter_supported?: boolean;
	request_parameter_supported?: boolean;
	request_uri_parameter_supported?: boolean;
	require_request_uri_registration?: boolean;
	op_policy_uri?: string;
	op_tos_uri?: string;
}
export interface OpenIdProviderConfigurationSessionFields {
	check_session_iframe?: string;
	end_session_endpoint?: string;
}
export interface OpenIdProviderConfiguration extends OpenIdProviderConfigurationMandatoryFields, OpenIdProviderConfigurationOptionalFields, OpenIdProviderConfigurationSessionFields {
}
export interface IExternalProviderPrimaryKey {
	provider: string;
	providerProtocol: string | 'openid' | 'oauth' | 'oauth2' | 'bot';
	providerURL: string;
	appID: string;
}
export interface IExternalProvider extends IExternalProviderPrimaryKey, IProfile {
	provider: string;
	providerProtocol: string | 'openid' | 'oauth' | 'oauth2' | 'bot';
	providerURL: string;
	providerFingerprint?: string;
	providerConfiguration?: any;
	appID: string;
	appSecret: string;
	scope: string;
	displayName: string;
	displayImageURL: string;
	description: string;
}
export declare class ExternalProviderModel extends VertexModel implements IExternalProvider {
	provider: string;
	providerProtocol: 'openid';
	providerURL: string;
	providerFingerprint: string;
	providerConfiguration: OpenIdProviderConfiguration;
	appID: string;
	appSecret: string;
	scope: string;
	displayName: string;
	displayImageURL: string;
	description: string;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	expireAt: Date;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
}
export declare class ExternalProviderCollection extends Collection<ExternalProviderModel> {
	constructor(db: Graph);
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
}
export interface IExternalUserIdentity {
	type: string;
	key?: string;
	value: string;
}
export interface IExternalUserKeyFields {
	uid: number | string;
	provider: string;
	providerProtocol: string | 'oauth' | 'oauth2' | 'bot';
}
export interface IExternalUserMandatoryFields extends IExternalUserKeyFields {
	uid: number | string;
	provider: string;
	providerProtocol: string | 'oauth' | 'oauth2' | 'bot';
	providerData: any;
	identities: Array<IExternalUserIdentity>;
}
export interface IExternalUserOptionalFields {
	user: ObjectId | null;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
}
export interface IExternalUser extends IVertex, IExternalUserMandatoryFields, IExternalUserOptionalFields, IMutable, IExpirable, IBlockable, IDeletable {
	uid: number | string;
	provider: string;
	providerProtocol: string | 'oauth' | 'oauth2' | 'bot';
	providerData: any;
	identities: Array<IExternalUserIdentity>;
	user: ObjectId | null;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	displayImageFile: ObjectId | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	expireAt: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	blocked: boolean;
	blockedAt: Date;
	blockedFrom: string;
	blockedBy: IdentityProfile;
	blockedReason: string;
	deleted: boolean;
	deletedAt: Date;
	deletedFrom: string;
	deletedBy: IdentityProfile;
	deletedReason: string;
}
export declare class ExternalUserModel extends VertexModel implements IExternalUser {
	uid: number | string;
	provider: string;
	providerProtocol: string | 'openid' | 'oauth' | 'oauth2' | 'bot';
	providerData: any;
	identities: Array<IExternalUserIdentity>;
	user: ObjectId | null;
	username: string;
	domain: string;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string;
	displayImageFile: ObjectId;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	blocked: boolean;
	blockedAt: Date;
	blockedFrom: string;
	blockedBy: IdentityProfile;
	blockedReason: string;
	deleted: boolean;
	deletedAt: Date;
	deletedFrom: string;
	deletedBy: IdentityProfile;
	deletedReason: string;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	expireAt: Date;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
}
export declare class ExternalUserCollection extends Collection<ExternalUserModel> {
	constructor(db: Graph);
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
}
export declare class SessionModel extends VertexModel implements IBlockable, IChainable, IDeletable {
	device: string;
	user: ObjectId;
	roles: Array<string>;
	account: string;
	token: ObjectId;
	tokens: Array<any>;
	previous: ObjectId;
	impersonated: boolean;
	impersonatedAt: Date;
	impersonatedFrom: string;
	impersonatedBy: IdentityProfile;
	impersonatedReason: string;
	expireAt: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
	blocked: boolean;
	blockedAt: Date;
	blockedFrom: string;
	blockedBy: IdentityProfile;
	blockedReason: string;
	deleted: boolean;
	deletedAt: Date;
	deletedFrom: string;
	deletedBy: IdentityProfile;
	deletedReason: string;
}
export declare class SessionCollection extends Collection<SessionModel> {
	constructor(db: Graph);
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
}
export declare class UserCollection extends Collection<UserModel> {
	constructor(db: Graph);
	static getIndices(): Array<any>;
	createIndices(): Promise<any>;
}
export interface ApfsFileHash {
	md5: Buffer;
	sha1: Buffer;
	sha256: Buffer;
	ipfs?: string;
}
export interface ApfsFileAccessToken {
	token: string;
	description?: string;
	notValidBefore: Date;
	notValidAfter?: Date;
	downloadQuota?: number;
	download: number;
}
export interface ApfsFileAccessStatistics {
	update: number;
	read: number;
}
export interface ApfsFileChunkInfo {
	_id: ObjectId;
	offset: number;
	byteLength: number;
}
export interface ApfsFileMetadata {
	originalname?: string;
	width?: number;
	height?: number;
	duration?: number;
}
export interface ApfsFileLike extends ApfsFileMetadata {
	data: Buffer;
	type: string;
	filename: string;
}
export declare class ApfsFile implements ApfsFileMetadata {
	filename: string;
	type: string;
	data: Buffer;
	byteLength: number;
	hash: ApfsFileHash;
	createdAt: Date;
	updatedAt?: Date;
	context: any;
	stats: ApfsFileAccessStatistics;
	originalname: string;
	width: number;
	height: number;
	duration: number;
}
export declare class ApfsFileChunkModel extends Model {
	hash: ApfsFileHash;
	type: string;
	byteLength: number;
	data: Buffer;
	dataOffset: number;
	dataByteLength: number;
	dataEncoding: 'binary';
	expireAt: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
}
export declare class ApfsFileChunkCollection extends Collection<ApfsFileChunkModel> {
	constructor(db: Graph);
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
}
export declare class ApfsFileModel extends VertexModel implements IChainable {
	toUrl(): string;
	path: string;
	name: string;
	type: string;
	byteLength: number;
	originalname: string;
	width: number;
	height: number;
	duration: number;
	hash: ApfsFileHash;
	chunks: Array<ApfsFileChunkInfo>;
	context: any;
	stats: ApfsFileAccessStatistics;
	tokens: Array<ApfsFileAccessToken>;
	previous: ObjectId;
	expireAt: Date;
	createdAt: Date;
	createdFrom: string;
	createdBy: IdentityProfile;
	updatedAt: Date;
	updatedFrom: string;
	updatedBy: IdentityProfile;
}
export declare class ApfsFileCollection extends Collection<ApfsFileModel> {
	constructor(db: Graph);
	static getIndices(): Array<Index>;
	createIndices(): Promise<any>;
}
export interface WfDomain extends WebFoundationDomain<WebFoundationSettings> {
}
export declare class AdminService extends WfService {
	createIndices(domain: WfDomain): Promise<Array<string>>;
}
export declare class ApfsService extends Resolvable<WebFoundationSettings> {
	parse(url: string): Partial<ApfsFileLike> | null;
	download(domain: WfDomain, url: string, overwrite?: boolean): Promise<ApfsFileModel | null>;
	save(domain: WfDomain, apfs: ApfsFileLike, overwrite?: boolean): Promise<ApfsFileModel>;
	load(domain: WfDomain, url: string): Promise<ApfsFile | null>;
}
export declare class UserService extends WfService {
	findOneById(domain: WfDomain, id: string | ObjectId | null | undefined): Promise<UserModel | null>;
	findOneByEmail(domain: WfDomain, emailAddress: string | null | undefined): Promise<UserModel | null>;
	insertOne(domain: WfDomain, user: IUser): Promise<UserModel>;
}
export declare class UserMutations {
	static insertOne(domain: WfDomain, user: IUser): Promise<UserModel>;
}
export declare class CurrentUser extends UserModel {
	static [TypeRegistry.Initializer](app: WfApp, ctx: WfCtx): Promise<CurrentUser | null>;
}
export declare class UserGetters {
	static findOneById(domain: WfDomain, id: any): Promise<UserModel | null>;
}
export declare class EmailGetters {
	static findOneByEmail(domain: WfDomain, emailAddress: string): Promise<EmailModel | null>;
}
export declare class SessionMutations {
	static CreateSession(domain: WfDomain, user: UserModel, allows: Array<string>, context?: any, previousSession?: SessionModel | null | undefined, impersonated?: boolean | undefined): Promise<SessionModel>;
	static CreateOrUpdateSession(domain: WfDomain, user: UserModel, allows: Array<string>, account: string, context?: any): Promise<SessionModel | null>;
	static ExtendSession(domain: WfDomain, context: any): Promise<SessionModel>;
	static RestoreSession(domain: WfDomain, sessionToRestore: SessionModel, context: any, reason: string): Promise<SessionModel | null>;
	static TerminateSession(domain: WfDomain, reason: string): Promise<SessionModel | null>;
}
export declare class WebAuthenticationService extends WfService {
	protected WebSessions: WebSessionService;
	protected Users: UserService;
	login(ctx: WfCtx, user: UserModel | ObjectId | string | null, allows?: Array<string>): Promise<SessionModel>;
	logout(ctx: WfCtx, reason?: string): Promise<SessionModel | null>;
	private update;
}
export declare class SessionGetters {
	static GetExternalUserFromUser(domain: WfDomain, user: UserModel, provider: string, providerProtocol: string): Promise<ExternalUserModel | null>;
}
export declare abstract class AuthenticatedController<T extends WebFoundationSettings> extends WebFoundationController<T> {
	__WebFoundationController_InitRequest(ctx: WfCtx, next: NextFunction): Promise<any>;
	protected getFailureURL(ctx: WfCtx, err: Error): Promise<string>;
}
export declare class CurrentSession extends SessionModel {
	static [TypeRegistry.Initializer](app: WebApplication<WebFoundationSettings>, ctx: WfCtx): Promise<SessionModel | null> | null;
}
export declare class ExternalProviderService extends WfService {
	findOrDiscoverExternalProvider(domain: WfDomain, profile: Partial<IExternalProvider>): Promise<ExternalProviderModel>;
}
export declare class ExternalProviderMutations {
	static findOrDiscoverExternalProvider(domain: WfDomain, profile: Partial<IExternalProvider>): Promise<ExternalProviderModel>;
	static findExternalProvider(domain: WfDomain, profile: Partial<IExternalProviderPrimaryKey>): Promise<ExternalProviderModel | null>;
	static discoverExternalProvider(domain: WfDomain, profile: Partial<IExternalProvider>): Promise<ExternalProviderModel>;
	static GetExternalProvider(domain: WfDomain, id: ObjectId | string): Promise<ExternalProviderModel | null>;
	private static discoverIdProviderConfiguration;
}
export interface IAuthenticationProfile extends IExternalUserMandatoryFields, IExternalUserOptionalFields {
	uid: number | string;
	provider: string;
	providerProtocol: string | 'oauth' | 'oauth2' | 'bot';
	providerData: any;
	identities: Array<IExternalUserIdentity>;
	username: string | null;
	domain: string | null;
	email: string | null;
	emailCertified: boolean;
	mobile: string | null;
	mobileCertified: boolean;
	displayName: string;
	displayImageURL: string | null;
	familyName: string | null;
	givenName: string | null;
	gender: string | null;
	dob: Date | null;
	status: string | null;
	bio: string | null;
	homepage: string | null;
	locale: string | null;
}
export declare class ExternalUserMutations {
	static FindExternalUser(domain: WfDomain, profile: IExternalUserKeyFields): Promise<ExternalUserModel | null>;
	static createOrUpdateExternalUser(domain: WfDomain, profile: Partial<IAuthenticationProfile>): Promise<ExternalUserModel>;
	static FindOrCreateUser(domain: WfDomain, externalUser: ExternalUserModel): Promise<UserModel>;
	static createNewUserFromExternalUser(domain: WfDomain, externalUser: ExternalUserModel): Promise<UserModel>;
	static CreateEmail(domain: WfDomain, externalUser: ExternalUserModel): Promise<EmailModel | null>;
}
export declare class ExternalUserService extends Resolvable<WebFoundationSettings> {
	upsertOne(domain: WfDomain, profile: Partial<IAuthenticationProfile> | null | undefined): Promise<ExternalUserModel>;
	createNewUserFromExternalUser(domain: WfDomain, externalUser: ExternalUserModel): Promise<UserModel>;
}
export declare class ExternalUserGetters {
	static FindExternalUser(domain: WfDomain, profile: IExternalUserMandatoryFields): Promise<ExternalUserModel | null>;
	static GetExternalUserFromUser(domain: WfDomain, user: UserModel, provider: string, providerProtocol: string): Promise<ExternalUserModel | null>;
}
export interface IAuthenticationToken {
	token_type: string;
	scope: string;
	expires_in: number;
	ext_expires_in: number;
	access_token: string;
	refresh_token: string;
	id_token: any;
}
export interface GoogleIdToken {
	azp: string;
	aud: string;
	sub: string;
	email: string;
	email_verified: boolean;
	at_hash: string;
	exp: number;
	iss: string;
	iat: number;
	name: string;
	picture: string;
	given_name: string;
	family_name: string;
	locale: string;
}
export interface GoogleToken extends IAuthenticationToken {
	id_token: GoogleIdToken;
}
export declare class GoogleService extends WfService {
	apfs: ApfsService;
	TrustedEmailDomains: string[];
	getProfile(domain: WfDomain, token: GoogleToken, provider: IExternalProvider): Promise<Partial<IAuthenticationProfile> | null>;
}
export interface MicrosoftIdToken {
	ver: string;
	iss: string;
	sub: string;
	aud: string;
	exp: number;
	iat: number;
	nbf: number;
	name: string;
	preferred_username: string;
	oid: string;
	email: string;
	tid: string;
	aio: string;
}
export interface MicrosoftToken extends IAuthenticationToken {
	id_token: MicrosoftIdToken;
}
export declare class MicrosoftService extends WfService {
	apfs: ApfsService;
	TrustedEmailDomains: string[];
	getProfile(domain: WfDomain, token: MicrosoftToken, provider: IExternalProvider): Promise<Partial<IAuthenticationProfile> | null>;
}
export interface IAuthenticator {
	idProvider: IExternalProvider;
	getAuthorizeURL(redirect_uri: string, state: any): string;
	getToken(redirect_uri: string, code: string): Promise<IAuthenticationToken>;
	refreshToken(token: IAuthenticationToken): Promise<any>;
	getProfile(token: IAuthenticationToken): Promise<Partial<IAuthenticationProfile> | null>;
	getFailureURL(err: Error): string;
	getSuccessURL(data: any): string;
}
export declare abstract class OpenIdAuthController<T extends WebFoundationSettings> extends WebFoundationController<T> {
	protected ExternalUsers: ExternalUserService;
	protected Users: UserService;
	abstract discover(ctx: WfCtx): Promise<IExternalProvider>;
	getAuthenticator(ctx: WfCtx): Promise<IAuthenticator>;
	abstract profile(domain: WfDomain, token: IAuthenticationToken, provider: IExternalProvider): Promise<Partial<IAuthenticationProfile> | null>;
	abstract authenticated(ctx: WfCtx, externalUser: ExternalUserModel, foundUsers: UserModel | null, loggedInUser?: UserModel | null): Promise<UserModel>;
	__OAuthController_Start(ctx: WfCtx): Promise<void>;
	__OAuthController_Callback(ctx: WfCtx): Promise<void>;
}